#include <arc/notify.h>
#include <arc/jobftpcontrol.h>

#include <globus_ftp_control.h>

JobFTPControl::JobFTPControl() throw(FTPControlError) : FTPControl() { }


JobFTPControl::~JobFTPControl() { }


std::string JobFTPControl::Submit(const URL& url,
                                  const std::string& rsl,
                                  bool disconnectafteruse,
                                  int timeout)
throw(FTPControlError, JobFTPControlError) {

	// dump rsl into temporary file
	char* filename = (char*)malloc(16);
	strncpy(filename, "/tmp/rsl.XXXXXX", 15);
	int fd = mkstemp(filename);
	if (fd==-1)	throw JobFTPControlError("Could not create temporary file.");
	write(fd, (void*)rsl.c_str(), rsl.size());
	close(fd);

	std::string resp;

	try {
		Connect(url, timeout);
		resp = SendCommand("CWD " + url.Path(), timeout);
		resp = SendCommand("CWD new", timeout);
	} catch(FTPControlError e) {
		throw JobFTPControlError(e.what());
	}

	std::string::size_type pos;
	if ( (pos = resp.find('"'))==std::string::npos)
		throw JobFTPControlError("Could not parse server response.");
	resp = resp.substr(pos+1);

	if ( (pos = resp.find('"'))==std::string::npos)
		throw JobFTPControlError("Could not parse server response.");
	jobid = resp.substr(0, pos);

	pos = jobid.rfind('/');
	if (pos==std::string::npos)
		throw JobFTPControlError("Invalid jobid returned by server");
	jobid = jobid.substr(pos+1);

	std::string uploadurl = url.str();
	if (uploadurl[uploadurl.length()-1]=='/') // remove trailing slash
		uploadurl.resize(uploadurl.length()-1);

	Upload(filename, uploadurl + "/new/job", false, timeout);
	if (disconnectafteruse) Disconnect(url, timeout);

	unlink(filename);
	free(filename);

	return uploadurl + "/" + jobid;
}


std::string JobFTPControl::Submit(const URL& url,
                                  const std::string& rsl,
                                  std::map<std::string, std::string>& files,
                                  bool disconnectafteruse,
                                  int timeout)
throw(FTPControlError, JobFTPControlError) {

	Connect(url, timeout);

	std::string uploadurl = url.str();
	if (uploadurl[uploadurl.length()-1]=='/') // remove trailing slash
		uploadurl.resize(uploadurl.length()-1);

	// Submit RSL and obtain ID
	Submit(URL(uploadurl), rsl, false, timeout);

	// Submit local files
	std::map<std::string, std::string>::iterator fileit;
	for (fileit = files.begin(); fileit != files.end(); fileit++) {

		notify(DEBUG) << "Uploading local-file from " << fileit->first
		              << " to " << fileit->second <<std::endl;

		try {
			Upload(fileit->first,
			       uploadurl + "/" + jobid + "/" + fileit->second,
			       false,
			       timeout);
		} catch(FTPControlError e) {
			std::cout << e.what() << std::endl;
		}
	}

	if (disconnectafteruse) Disconnect(url, timeout);

	return uploadurl + "/" + jobid;
}


void JobFTPControl::Cancel(const std::string& jobid,
                           bool disconnectafteruse,
                           int timeout)
throw(JobFTPControlError, FTPControlError) {

	URL url(jobid);
	std::string path = url.Path();

	std::string::size_type pos = path.rfind('/');
	if (pos==std::string::npos || pos==0)
		throw JobFTPControlError("Illegal jobid specified.");

	std::string jobpath = path.substr(0, pos);
	std::string jobidnum = path.substr(pos+1);

	Connect(url, timeout);
	SendCommand("CWD " + jobpath, timeout);
	SendCommand("DELE " + jobidnum, timeout);
	if (disconnectafteruse) Disconnect(url, timeout);
}


void JobFTPControl::Clean(const std::string& jobid,
                          bool disconnectafteruse,
                          int timeout)
throw(JobFTPControlError, FTPControlError) {

	URL url(jobid);
	std::string path = url.Path();

	std::string::size_type pos = path.rfind('/');
	if (pos==std::string::npos || pos==0)
		throw JobFTPControlError("Illegal jobid specified.");

	std::string jobpath = path.substr(0, pos);
	std::string jobidnum = path.substr(pos+1);

	Connect(url, timeout);
	SendCommand("CWD " + jobpath, timeout);
	SendCommand("RMD " + jobidnum, timeout);
	if (disconnectafteruse) Disconnect(url, timeout);
}


void JobFTPControl::RenewCreds(const std::string& jobid,
                               bool disconnectafteruse,
                               int timeout)
throw(JobFTPControlError, FTPControlError) {

	URL url(jobid);
	std::string path = url.Path();

	std::string::size_type pos = path.rfind('/');
	if (pos==std::string::npos || pos==0)
		throw JobFTPControlError("Illegal jobid specified.");

	std::string jobpath = path.substr(0, pos);
	std::string jobidnum = path.substr(pos+1);

	Connect(url, timeout);
	SendCommand("CWD " + jobpath, timeout);
	SendCommand("CWD " + jobidnum, timeout);
	if (disconnectafteruse) Disconnect(url, timeout);
}


void CancelJob(const std::string& jobid) throw(JobFTPControlError, FTPControlError) {
	JobFTPControl ftpc;
	ftpc.Cancel(jobid);
}


void CleanJob(const std::string& jobid) throw(JobFTPControlError, FTPControlError) {
	JobFTPControl ftpc;
	ftpc.Clean(jobid);
}


void RenewCreds(const std::string& jobid) throw(JobFTPControlError, FTPControlError) {
	JobFTPControl ftpc;
	ftpc.RenewCreds(jobid);
}

