#include <iostream>
#include <string>

#include <arc/url.h>

void TestURL(const URL& url) {

	std::cout << url << std::endl;
	std::cout << url.CanonicalURL() << std::endl;

	if (!url.Protocol().empty())
		std::cout << "Protocol: " << url.Protocol() << std::endl;
	if (!url.Username().empty())
		std::cout << "Username: " << url.Username() << std::endl;
	if (!url.Passwd().empty())
		std::cout << "Passwd: " << url.Passwd() << std::endl;
	if (!url.Host().empty())
		std::cout << "Host: " << url.Host() << std::endl;
	if (url.Port() != -1)
		std::cout << "Port: " << url.Port() << std::endl;
	if (!url.Path().empty())
		std::cout << "Path: " << url.Path() << std::endl;
	if (url.Protocol() == "ldap")
		std::cout << "BaseDN: " << url.BaseDN() << std::endl;
	if (!url.HTTPOptions().empty()) {
		std::cout << "HTTP Options: " << std::endl;
		for (std::map<std::string, std::string>::const_iterator it =
		     url.HTTPOptions().begin(); it != url.HTTPOptions().end(); it++) {
			std::cout << "   " << it->first
			          << " = " << it->second << std::endl;
		}
	}
	if (!url.Options().empty()) {
		std::cout << "URL Options: " << std::endl;
		for (std::map<std::string, std::string>::const_iterator it =
		     url.Options().begin(); it != url.Options().end(); it++) {
			std::cout << "   " << it->first
			          << " = " << it->second << std::endl;
		}
	}
	if (!url.Locations().empty()) {
		std::cout << std::endl;
		std::cout << "URL Locations: ------------- Begin" << std::endl;
		std::cout << std::endl;
		for (std::list<URLLocation>::const_iterator it =
		     url.Locations().begin(); it != url.Locations().end(); it++) {
			if (!it->Name().empty())
				std::cout << "Name: " << it->Name() << std::endl;
			TestURL (*it);
		}
		std::cout << "URL Locations: --------------- End" << std::endl;
	}
	std::cout << std::endl;
}


void TestURL(const std::string& u) {

	std::cout << u << std::endl;

	URL url(u);

	TestURL(url);
}


int main( )
{
	std::string gsiftpurl("gsiftp://hathi.hep.lu.se/public/test.txt");
	TestURL(gsiftpurl);

	std::string ldapurl("ldap://grid.uio.no/o=grid/mds-vo-name=local");
    TestURL(ldapurl);

	std::string httpurl("http://www.nordugrid.org/monitor.php?debug=2&sort=yes");
	TestURL(httpurl);

	std::string fileurl("file:///home/grid/runtime/TEST-ATLAS-8.0.5");
	TestURL(fileurl);

	std::string ldapurl2("ldap://grid.uio.no/mds-vo-name=local, o=grid");
    TestURL(ldapurl2);

	std::string opturl("gsiftp://hathi.hep.lu.se;ftpthreads=10;upload=yes/public/test.txt");
	TestURL(opturl);

	std::string ftpurl2("ftp://user:secret@ftp.nordugrid.org/pub/files/guide.pdf");
	TestURL(ftpurl2);

	std::string rcurl("rc://grid.uio.no/test.txt");
	TestURL(rcurl);

	std::string rcurl2("rc://hagrid|sigrid@grid.uio.no/test.txt");
	TestURL(rcurl2);

	std::string rcurl3("rc://hagrid;local=yes|sigrid;threads=10;local=no@grid.uio.no;local=no/test.txt");
	TestURL(rcurl3);

	std::string rlsurl("rls://gridsr3.nbi.dk/test.txt");
	TestURL(rlsurl);

	std::string rlsurl2("rls://gsiftp://hagrid.it.uu.se/storage/test.txt|http://www.nordugrid.org/files/test.txt@gridsrv3.nbi.dk/test.txt");
	TestURL(rlsurl2);

	std::string rlsurl3("rls://gsiftp://hagrid.it.uu.se;threads=10/storage/test.txt|http://www.nordugrid.org;cache=no/files/test.txt@gridsrv3.nbi.dk;local=yes/test.txt");
	TestURL(rlsurl3);
}
