#ifndef __HTTPSD_FIREMAN_CLIENT_H__
#define __HTTPSD_FIREMAN_CLIENT_H__

#include <stdsoap2.h>
#include <string>
#include <list>

#include "../client/client.h"

#include "../../datamove/datapoint.h"

/// Minimal set of operations on Fireman database. Mostly needed to
/// implement corresponding DataPoint class.
class FiremanClient {
 private:
  struct soap soap;
  HTTP_ClientSOAP* c;
  int timeout;
//  bool acquire(SRMRequest& req,std::list<string>& urls);
 public:
  FiremanClient(const char* base_url);
  ~FiremanClient(void);
  bool connect(void);
  bool disconnect(void);
  /// Create file
  bool add(const char* name,
       unsigned long long int size,const string& checksum,time_t time);
  /// Create file with associated replicas
  bool add(const char* name,
       unsigned long long int size,const string& checksum,time_t time,
       const std::list<string>& urls);
  /// Add replicas to file
  bool add(const char* name,
       const std::list<string>& urls);
  /// Get minimal associated metadata
  bool info(const char* name,
       unsigned long long int& size,string& checksum,time_t& time,
       DataPoint::FileInfo::Type& type);
  /// Get list of file replicas and minimal associated metadata
  bool info(const char* name,
       unsigned long long int& size,string& checksum,time_t& time,
       DataPoint::FileInfo::Type& type,
       std::list<string>& urls);
  /// Get list of file replicas
  bool info(const char* name,
       std::list<string>& urls);
  /// Remove file and all associated info (attributes, replicas)
  bool remove(const char* name);
  /// Remove associated replicas
  bool remove(const char* name,const std::list<string>& urls);
  bool list(const char* path,std::list<string>& names);
  operator bool(void) { return c; };
  bool operator!(void) { return !c; };
};

class DataPointFireman: public DataPointMeta {
 private:
  FiremanClient* c;
 protected:
  virtual bool process_meta_url(void);
 public:
  DataPointFireman(const char* url);
  ~DataPointFireman(void);
  static DataPoint* CreateInstance(const char* url);
  virtual bool meta_resolve(bool source);
  virtual bool meta_preregister(bool replication,bool force = false);
  virtual bool meta_postregister(bool replication,bool failure);
  virtual bool meta_preunregister(bool replication);
  virtual bool meta_unregister(bool all);
  virtual bool list_files(std::list<DataPoint::FileInfo> &files,bool resolve = true);};


#endif // __HTTPSD_FIREMAN_CLIENT_H__
