#ifndef FLO_COMMON_H
#define FLO_COMMON_H

#include "../../misc/log_time.h"
#include "../../misc/condition.h"

class pSafeObject;

class SafeObject {
 friend class pSafeObject;
 private:
  int p_counter_;
  LockSimple p_lock_;
  void acquire(void) {
    p_lock_.block();
    ++p_counter_;
    odlog(4)<<"SafeObject: acquire: counter: "<<p_counter_<<endl;
    odlog(4)<<"SafeObject: "<<this<<endl;
    p_lock_.unblock();
  };
  void release(void) {
    p_lock_.block();
    int n = --p_counter_;
    odlog(4)<<"SafeObject: release: counter: "<<p_counter_<<endl;
    odlog(4)<<"SafeObject: "<<this<<endl;
    p_lock_.unblock();
    if(n == 0) delete this;
  };
 public:
  SafeObject(void):p_counter_(0) {
    odlog(4)<<"SafeObject: constructor: "<<this<<endl;
  };
  virtual ~SafeObject(void);
};

class pSafeObject {
 protected:
  SafeObject* object_;
 public:
  pSafeObject(void) {
    odlog(4)<<"pSafeObject: constructor: NULL"<<endl;
    odlog(4)<<"pSafeObject: "<<this<<endl;
    object_=NULL;
  };
  pSafeObject(SafeObject& o) {
    odlog(4)<<"pSafeObject: constructor: object"<<endl;
    odlog(4)<<"pSafeObject: "<<this<<endl;
    o.acquire(); object_=&o;
  };
  pSafeObject(SafeObject* o) {
    odlog(4)<<"pSafeObject: constructor: pointer"<<endl;
    odlog(4)<<"pSafeObject: "<<this<<endl;
    if(o) o->acquire(); object_=o;
  };
  pSafeObject(const pSafeObject& p) {
    odlog(4)<<"pSafeObject: constructor: safe pointer"<<endl;
    odlog(4)<<"pSafeObject: "<<this<<endl;
    if(p.object_) p.object_->acquire(); object_=p.object_;
  };
  ~pSafeObject(void) {
    odlog(4)<<"pSafeObject: destructor"<<endl;
    odlog(4)<<"pSafeObject: "<<this<<endl;
    if(object_) object_->release();
  };
  SafeObject& operator*(void) const { return *object_; };
  SafeObject* operator->(void) const { return object_; };
  void operator=(SafeObject& o) {
    odlog(4)<<"pSafeObject: =: object"<<endl;
    odlog(4)<<"pSafeObject: "<<this<<endl;
    o.acquire();
    if(object_) object_->release();
    object_=&o;
  };
  void operator=(SafeObject* o) {
    odlog(4)<<"pSafeObject: =: pointer"<<endl;
    odlog(4)<<"pSafeObject: "<<this<<endl;
    if(o) o->acquire();
    if(object_) object_->release();
    object_=o;
  };
  void operator=(const pSafeObject& p) {
    odlog(4)<<"pSafeObject: =: safe pointer"<<endl;
    odlog(4)<<"pSafeObject: "<<this<<endl;
    if(p.object_) p.object_->acquire();
    if(object_) object_->release();
    object_=p.object_;
  };
  operator bool(void) const { return (object_!=NULL); };
  bool operator!(void) const { return (object_==NULL); };
  bool operator==(const pSafeObject& p) const { return (object_ == p.object_); };
  bool operator!=(const pSafeObject& p) const { return (object_ != p.object_); };
  bool operator==(const SafeObject& o) const { return (object_ == &o); };
  bool operator!=(const SafeObject& o) const { return (object_ != &o); };
};

#endif // FLO_COMMON_H
