#include "../../std.h"

#include "../../misc/inttostring.h"
#include "../../misc/log_time.h"

#include "jobs_client.h"

JobHTTPControl::JobHTTPControl(void):url_(NULL),client_(NULL) {
}

JobHTTPControl::~JobHTTPControl(void) {
  if(client_) delete client_;
}

JobHTTPControl::operator bool(void) {
  return (client_ != NULL);
}

bool JobHTTPControl::operator!(void) {
  return (client_ == NULL);
}


string JobHTTPControl::Submit(const URL& url,const string& rsl,bool disconnectafteruse,int timeout) {
  jobid_="";
  if(!url) return "";
  if(url_) {
    if((url_.Proto() == url.Proto()) &&
       (url_.Host() == url.Host()) &&
       (url_.Port() == url.Port())) {
      url_=url;
    } else { // reinitialize for different server
      url_=url; if(client_) delete client_; client_=NULL;
    };
  } else { // initialize for first connection
    url_=url; if(client_) delete client_; client_=NULL;
  };
  if(client_ == NULL) {
    string soap_url = url_.Proto()+"://"+url_.Host()+":"+
                      inttostring(url_.Port());
    client_ = new HTTP_ClientSOAP(soap_url.c_str(),&soap_);
    soap_.namespaces=jobs_soap_namespaces;
  } else {
    client_->reset();
  };
  // connect to server if not yet connected
  if(client_->connect() != 0) {
    odlog(1)<<"Failed to connect to "<<url_<<endl;
    return "";
  };
  int soap_err = SOAP_OK;
  nj__definitionType* definition = soap_new_nj__definitionType(&soap_,1);
  if(definition == NULL) { return ""; };
  definition->description=(char*)(rsl.c_str());
  __infosType r;
  if((soap_err=soap_call_njf__create(&soap_,client_->SOAP_URL(),"create",
                                              1,definition,r)) != SOAP_OK) {
    odlog(1)<<"Failed to submit job"<<endl;
    if(LogTime::level > 0) soap_print_fault(&soap_,stderr);
    client_->disconnect(); client_->reset(); return "";
  } else if((r.__size_job != 1) || (r.job == NULL)) {
    odlog(1)<<"No proper response received"<<endl;
    client_->disconnect(); client_->reset(); return "";
  } else if((r.job[0].failure_code!=NULL) && (r.job[0].failure_code[0]!=0)) {
    if(r.job[0].failure_reason) {
      odlog(1)<<"Submission failed: code "<<r.job[0].failure_code[0]<<
                ": "<<r.job[0].failure_reason<<endl;
    } else {
      odlog(1)<<"Submission failed: code "<<r.job[0].failure_code[0]<<endl;
    };
    client_->disconnect(); client_->reset(); return "";
  } else if(r.job[0].id == NULL) {
    odlog(1)<<"Submission did not return job's ID"<<endl;
    client_->disconnect(); client_->reset(); return "";
  } else if(r.job[0].url == NULL) {
    odlog(1)<<"Submission did not return job's URL"<<endl;
    client_->disconnect(); client_->reset(); return "";
  };
  jobid_=r.job[0].id; joburl_=r.job[0].url;
  if(disconnectafteruse) client_->disconnect();
  client_->reset();
  return jobid_;
}

string JobHTTPControl::Submit(const URL& url,const string& rsl,map<string,string>& files,bool disconnectafteruse = true,int timeout = TIMEOUT) {
  Submit(url,rsl,false,timeout);
  if(jobid_.length() == 0) return "";
  std::map<std::string, std::string>::iterator fileit;
  for(fileit=files.begin();fileit!=files.end();++fileit) {
    unsigned char buffer[1024*1024];
    unsigned long long int offset = 0;
    struct stat st;
    if(stat(fileit->first.c_str(),&st) != 0) {
      odlog(0)<<"Can't find file "<<fileit->first<<endl;
      client_->reset(); client_->disconnect(); jobid_=""; return "";
    };
    int fd = open(fileit->first.c_str(),O_RDONLY);
    if(fd == -1) {
      odlog(0)<<"Can't open file "<<fileit->first<<endl;
      client_->reset(); client_->disconnect(); jobid_=""; return "";
    };
    string http_path=URL(joburl_.c_str()).Path()+"/"+fileit->second;
    for(;;) {
      ssize_t l = read(fd,buffer,sizeof(buffer));
      if(l == 0) break;
      if(l == -1) {
        odlog(1)<<"Failed to read from file "<<fileit->first<<endl;
        client_->reset(); client_->disconnect(); jobid_=""; return "";
      };
      if(client_->connect() != 0) {
        odlog(1)<<"Failed to connect to "<<url_<<endl;
        client_->reset(); client_->disconnect(); jobid_=""; return "";
      };
      if(client_->PUT(http_path.c_str(),offset,l,buffer,st.st_size,true) != 0) {
        odlog(1)<<"Failed to store file "<<fileit->first<<" at "<<joburl_<<endl;
        client_->reset(); client_->disconnect(); jobid_=""; return "";
      };
      offset+=l;
    };
  };
  client_->reset();
  if(disconnectafteruse) client_->disconnect();
  return jobid_;
}

bool JobHTTPControl::Cancel(const std::string& jobid,bool disconnectafteruse,int timeout) {
  odlog(1)<<"Job cancel through WS interface is not implemented yet."<<endl;
  return false;
}

bool JobHTTPControl::Clean(const std::string& jobid,bool disconnectafteruse,int timeout) {
  odlog(1)<<"Job clean through WS interface is not implemented yet."<<endl;
  return false;
}

bool JobHTTPControl::RenewCreds(const std::string& jobid,bool disconnectafteruse,int timeout) {
  odlog(1)<<"Credentials renew through WS interface is not implemented yet."<<endl;
  return false;
}

